import os

FN_FOLDER = os.getenv("KESS_FN_FOLDER", "fn")
LIB_FOLDER = os.getenv("KESS_LIB_FOLDER", "lib")
RUNTIME_NAME  = os.getenv("KESS_RUNTIME_NAME", "")
RUNTIME_PREFIX = os.getenv("KESS_RUNTIME_PREFIX", "")
